#! /usr/bin/env python3
from rich import print_json
import typer
from rich.console import Console
from rich.table import Table
import json
import httpx

from snyk_tags import __app_name__, __version__

app = typer.Typer()
console = Console()
"""
List all the different project types and attribute types
"""


# List all project types command
@app.command(help="List all Snyk project types")
def types():
    snykcmd = typer.style(
        "snyk-tags target tag or snyk-tags tag custom",
        bold=True,
        fg=typer.colors.MAGENTA,
    )
    typer.echo(f"These are all the attribute types you can apply with {snykcmd}")
    table = Table("Snyk IaC", "Snyk Open Source", "Snyk Container", "Snyk Code")
    table.add_row("terraformconfig", "maven", "dockerfile", "sast")
    table.add_row("dockerfile", "npm", "apk", "")
    table.add_row("terraformplan", "nuget", "deb", "")
    table.add_row("k8sconfig", "gradle", "rpm", "")
    table.add_row("helmconfig", "pip", "linux", "")
    table.add_row("cloudformationconfig", "yarn", "", "")
    table.add_row("armconfig", "gomodules", "", "")
    table.add_row("", "rubygems", "", "")
    table.add_row("", "composer", "", "")
    table.add_row("", "sbt", "", "")
    table.add_row("", "golangdep", "", "")
    table.add_row("", "cocoapods", "", "")
    table.add_row("", "poetry", "", "")
    table.add_row("", "govendor", "", "")
    table.add_row("", "cpp", "", "")
    table.add_row("", "yarn-workspace", "", "")
    table.add_row("", "hex", "", "")
    table.add_row("", "paket", "", "")
    table.add_row("", "golang", "", "")
    console.print(table)


# List Attributes Command
@app.command(help="List all Snyk attribute")
def attributes():
    snykcmd = typer.style(
        "snyk-tags target attributes", bold=True, fg=typer.colors.MAGENTA
    )
    typer.echo(f"These are all the attribute types you can apply with {snykcmd}")
    table = Table("Criticality", "Environment", "Lifecycle")
    table.add_row("critical", "frontend", "production")
    table.add_row("high", "backend", "development")
    table.add_row("medium", "internal", "sandbox")
    table.add_row("low", "external", "")
    table.add_row("", "mobile", "")
    table.add_row("", "saas", "")
    table.add_row("", "onprem", "")
    table.add_row("", "hosted", "")
    table.add_row("", "distributed", "")
    console.print(table)


# Functions for tags listing command
def create_client(token: str, tenant: str) -> httpx.Client:
    base_url = (
        f"https://api.{tenant}.snyk.io/v1"
        if tenant in ["eu", "au", "us"]
        else "https://api.snyk.io/v1"
    )
    headers = {"Authorization": f"token {token}"}
    return httpx.Client(base_url=base_url, headers=headers)


# Get the tags from a group
def find_tags(token: str, group_id: str, jsonflag: bool, tenant: str) -> tuple:
    with create_client(token=token, tenant=tenant) as client:
        req = client.get(f"group/{group_id}/tags")
        group = client.get(f"group/{group_id}/orgs", timeout=None).json()
        group_name = group["name"]
        if req.status_code == 200:
            if jsonflag is False:
                print(f"These are the tags in Group: {group_name}")
                table = Table("Key", "Value")
                for tags in req.json().get("tags"):
                    key = tags.get("key")
                    value = tags.get("value")
                    table.add_row(key, value)
                console.print(table)
            elif jsonflag is True:
                print(json.dumps(req.json()))
        if req.status_code == 404:
            print(f"Group {group_name} not found. Error message: {req.json()}.")
        return req.status_code, req.json()


# List existing tags in a Group Command
@app.command(help="List all existing tags in a Group")
def tags(
    group_id: str = typer.Option(
        ...,  # Default value of comamand
        envvar=["GROUP_ID"],
        help="Specify the Group you want to see the tags from",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with Group admin access",
        envvar=["SNYK_TOKEN"],
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    json: bool = typer.Option(
        False,
        "--json",  # Default value of comamand
        help=f"Output into json format (default is a table), use --json to change output.",
    ),
):
    find_tags(
        snyktkn,
        group_id,
        json,
        tenant=tenant,
    )



#! /usr/bin/env python3

import logging

import httpx
import typer
from rich import print
from snyk import SnykClient
from typing import Dict

from snyk_tags import __app_name__, __version__, attribute, github

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

logging.getLogger("httpx").setLevel(logging.WARNING)

app = typer.Typer()
app.add_typer(
    github.app,
    name="github",
    help="Use GitHub metadata such as CODEOWNERS and GitHub Topics to add to Snyk projects",
)


# Reach to the API and generate tokens
def create_client(token: str, tenant: str) -> httpx.Client:
    base_url = (
        f"https://api.{tenant}.snyk.io/v1"
        if tenant in ["eu", "au", "us"]
        else "https://api.snyk.io/v1"
    )
    headers = {"Authorization": f"token {token}"}
    return httpx.Client(base_url=base_url, headers=headers)


# Apply tags to a specific project
def apply_tag_to_project(
    client: httpx.Client,
    org_id: str,
    project_id: str,
    tag: str,
    key: str,
    project_name: str,
) -> tuple:
    tag_data = {
        "key": key,
        "value": tag,
    }

    req = client.post(
        f"org/{org_id}/project/{project_id}/tags", data=tag_data, timeout=None
    )

    if req.status_code == 200:
        logging.info(f"Successfully added {tag_data} tags to Project: {project_name}.")
    elif req.status_code == 422:
        logging.warning(
            f"Tag {key}:{tag} is already applied for Project: {project_name}."
        )
    elif req.status_code == 404:
        logging.error(
            f"Project not found, likely a READ-ONLY project. Project: {project_name}. Error message: {req.json()}."
        )
    return req.status_code, req.json()


# Tagging loop
def apply_tags_to_projects(
    token: str,
    org_ids: list,
    name: str,
    tag: str,
    key: str,
    tenant: str,
    filters: Dict[str, any],
) -> None:
    with create_client(token=token, tenant=tenant) as client:
        for org_id in org_ids:
            base_url = (
                f"https://api.{tenant}.snyk.io/rest"
                if tenant in ["eu", "au", "us"]
                else "https://api.snyk.io/rest"
            )
            client_v3 = SnykClient(
                token=token, url=base_url, version="2023-08-31~experimental"
            )
            params = {"limit": 100, **filters}
            projects = client_v3.get_rest_pages(
                f"/orgs/{org_id}/projects", params=params
            )

            badname = 0
            rightname = 0
            for project in projects:
                if (
                    project["attributes"]["name"] == name
                    or project["attributes"]["name"].startswith(name + "(")
                    or project["attributes"]["name"].startswith(name + ":")
                ):
                    apply_tag_to_project(
                        client=client,
                        org_id=org_id,
                        project_id=project["id"],
                        tag=tag,
                        key=key,
                        project_name=project["attributes"]["name"],
                    )
                    rightname = 1
                else:
                    badname = 1
            if badname == 1 and rightname == 0:
                print(
                    f"[bold red]{name}[/bold red] is not a valid target, please check it is a target within the organization e.g. [bold blue]snyk-labs/snyk-goof[/bold blue]"
                )


# Coloured variables for output
crit = typer.style("critical, high, medium, low", bold=True, fg=typer.colors.MAGENTA)
enviro = typer.style(
    "frontend, backend, internal, 	external, mobile, saas, onprem, hosted, distributed",
    bold=True,
    fg=typer.colors.MAGENTA,
)
life = typer.style(
    "production, development, sandbox", bold=True, fg=typer.colors.MAGENTA
)
repoexample = typer.style("'snyk-labs/nodejs-goof'", bold=True, fg=typer.colors.MAGENTA)


@app.command(help=f"Apply a custom tag to a target, for example {repoexample}")
def tag(
    org_id: str = typer.Option(
        ...,  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify the Organization ID where you want to apply the tag",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    target: str = typer.Option(
        ...,  # Default value of comamand
        help=f"Name of the target, for example {repoexample}",
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    tagKey: str = typer.Option(
        ..., help="Tag key: identifier of the tag"  # Default value of comamand
    ),
    tagValue: str = typer.Option(
        ..., help="Tag value: value of the tag"  # Default value of comamand
    ),
):
    typer.secho(
        f"\nAdding the tag key {tagKey} and tag value {tagValue} to projects within {target} for easy filtering via the UI",
        bold=True,
        fg=typer.colors.MAGENTA,
    )
    apply_tags_to_projects(snyktkn, [org_id], target, tagValue, tagKey, tenant=tenant)


# Collection command to apply the attributes to the collection
@app.command(help=f"Apply attributes to a target, for example {repoexample}")
def attributes(
    org_id: str = typer.Option(
        ...,  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify the Organization ID where you want to apply the attributes",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    target: str = typer.Option(
        ...,  # Default value of comamand
        help=f"Name of the project collection, for example {repoexample}",
    ),
    criticality: str = typer.Option(
        "", help=f"Criticality attribute: {crit}"  # Default value of comamand
    ),
    environment: str = typer.Option(
        "", help=f"Environment attribute: {enviro}"  # Default value of comamand
    ),
    lifecycle: str = typer.Option(
        "", help=f"Lifecycle attribute: {life}"  # Default value of comamand
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
):
    typer.secho(
        f"\nAdding the attributes {criticality}, {environment} and {lifecycle} to projects within {target} for easy filtering via the UI",
        bold=True,
        fg=typer.colors.MAGENTA,
    )
    attribute.apply_attributes_to_projects(
        snyktkn,
        [org_id],
        target,
        [criticality],
        [environment],
        [lifecycle],
        tenant=tenant,
    )



#! /usr/bin/env python3

import json
import logging

import httpx
import typer
from rich import print
from snyk import SnykClient
from typing import Dict

from snyk_tags import __app_name__, __version__

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

logging.getLogger("httpx").setLevel(logging.WARNING)

app = typer.Typer()


# Reach to the API and generate tokens
def create_client(token: str, tenant: str) -> httpx.Client:
    base_url = (
        f"https://api.{tenant}.snyk.io/v1"
        if tenant in ["eu", "au", "us"]
        else "https://api.snyk.io/v1"
    )
    headers = {"Authorization": f"token {token}", "Content-Type": "application/json"}
    return httpx.Client(base_url=base_url, headers=headers)


# Apply attributes to a specific project
def apply_attributes_to_project(
    client: httpx.Client,
    org_id: str,
    project_id: str,
    criticality: list,
    environment: list,
    lifecycle: list,
    project_name: str,
) -> tuple:
    if criticality.count("") == 1:
        criticality.remove("")

    if environment.count("") == 1:
        environment.remove("")

    if lifecycle.count("") == 1:
        lifecycle.remove("")

    attribute_data = {
        "criticality": criticality,
        "environment": environment,
        "lifecycle": lifecycle,
    }

    req = client.post(
        f"org/{org_id}/project/{project_id}/attributes",
        data=json.dumps(attribute_data).replace("'", '"'),
        timeout=None,
    )

    attribute_data = typer.style(attribute_data, bold=True, fg=typer.colors.MAGENTA)
    criticality = typer.style(criticality, bold=True, fg=typer.colors.MAGENTA)
    environment = typer.style(environment, bold=True, fg=typer.colors.MAGENTA)
    lifecycle = typer.style(lifecycle, bold=True, fg=typer.colors.MAGENTA)

    if req.status_code == 200:
        logging.info(
            f"Successfully added {criticality},{environment},{lifecycle} attributes to Project: {project_name}."
        )
    elif req.status_code == 422:
        logging.warning(
            f"Data {attribute_data} cannot be processed, make sure you have written the correct values (refer to help or Readme) and that they are in low caps. Error message: {req.json()}."
        )
    elif req.status_code == 404:
        logging.error(
            f"Project not found, likely a READ-ONLY project. Project: {project_name}. Error message: {req.json()}."
        )
    elif req.status_code == 500:
        logging.error(
            f"Error message: {req.json()}. Please contact eric.fernandez@snyk.io."
        )
    return req.status_code, req.json()


# Apply attributes to projects within a collection
def apply_attributes_to_projects(
    token: str,
    org_ids: list,
    name: str,
    criticality: list,
    environment: list,
    lifecycle: list,
    tenant: str,
    filters: Dict[str, any],
) -> None:
    with create_client(token=token, tenant=tenant) as client:
        for org_id in org_ids:
            base_url = (
                f"https://api.{tenant}.snyk.io/rest"
                if tenant in ["eu", "au", "us"]
                else "https://api.snyk.io/rest"
            )
            client_v3 = SnykClient(
                token=token, url=base_url, version="2023-08-31~experimental"
            )
            params = {"limit": 100, **filters}
            projects = client_v3.get_rest_pages(
                f"/orgs/{org_id}/projects", params=params
            )

            badname = 0
            rightname = 0

            for project in projects:
                if project["attributes"]["name"].startswith(name):
                    apply_attributes_to_project(
                        client=client,
                        org_id=org_id,
                        project_id=project["id"],
                        criticality=criticality,
                        environment=environment,
                        lifecycle=lifecycle,
                        project_name=project["attributes"]["name"],
                    )
                    rightname = 1
                else:
                    badname = 1
            if badname == 1 and rightname == 0:
                print(
                    f"[bold red]{name}[/bold red] is not a valid target, please check it is a target within the organization e.g. [bold blue]snyk-labs/snyk-goof[/bold blue]"
                )



#! /usr/bin/env python3

import csv
from enum import Enum
import json
import logging
import sys

import typer
from rich import print as rich_print

from snyk_tags import __app_name__, __version__
from snyk_tags.lib.api import Api
from snyk_tags.lib.component.rules import parse_rules, project_matcher

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

app = typer.Typer()


class FormatType(str, Enum):
    log = "log"
    csv = "csv"
    json = "json"


class CsvFormatter:
    def __init__(self):
        self.wrote_header = False
        self.w = csv.writer(sys.stdout)

    def print(
        self,
        action: str,
        component: str,
        dry_run: bool,
        exclusive: bool,
        remove: bool,
        project: any,
    ):
        if not self.wrote_header:
            self.w.writerow(
                [
                    "action",
                    "mode",
                    "component",
                    "project.id",
                    "project.name",
                    "project.origin",
                    "project.target.display_name",
                    "project.target.url",
                    "project.target_reference",
                ]
            )
            self.wrote_header = True
        self.w.writerow(
            [
                "{}{}".format(dry_run and "would " or "", action),
                "{}{}".format(
                    remove and "remove" or "add",
                    exclusive and " exclusive" or "",
                ),
                component,
                project.get("id"),
                project.get("name"),
                project.get("origin"),
                project.get("target", {}).get("display_name"),
                project.get("target", {}).get("url"),
                project.get("target_reference"),
            ]
        )
        sys.stdout.flush()


class JsonFormatter:
    def print(
        self,
        action: str,
        component: str,
        dry_run: bool,
        exclusive: bool,
        remove: bool,
        project: any,
    ):
        print(
            json.dumps(
                {
                    "action": "{}{}".format(dry_run and "would " or "", action),
                    "mode": "{}{}".format(
                        remove and "remove" or "add",
                        exclusive and " exclusive" or "",
                    ),
                    "component": component,
                    "project": project,
                }
            )
        )


class LogFormatter:
    def print(
        self,
        action: str,
        component: str,
        dry_run: bool,
        exclusive: bool,
        remove: bool,
        project: any,
    ):
        rich_print(
            f"""{'{}{}'.format(dry_run and "would " or "", action)
                } "component:{component}" in project id="{project["id"]}" name="{project["name"]}\""""
        )


@app.command(help=f"Manage software component project tags")
def tag(
    rules: str = typer.Argument(...),
    org_id: str = typer.Option(
        ...,  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify the Organization ID where you want to apply the tag",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    dry_run: bool = typer.Option(
        default=False,
        help="Dry run",
    ),
    remove: bool = typer.Option(
        default=False,
        help="Remove matching component tags defined by rules",
    ),
    exclusive: bool = typer.Option(
        default=False,
        help="Remove all other component tags from projects. When used in combination with --remove, all component tags are removed from matching projects.",
    ),
    format: FormatType = typer.Option(
        default=FormatType.log,
        help="Output format, one of: log, csv, json",
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
):
    if format == "csv":
        fmtr = CsvFormatter()
    elif format == "json":
        fmtr = JsonFormatter()
    else:
        fmtr = LogFormatter()

    with open(rules, "r") as f:
        rules_doc = parse_rules(f)
        (match_fn, context) = project_matcher(rules_doc)
        client = Api(
            snyktkn,
            v1_url=(
                f"https://api.{tenant}.snyk.io/v1"
                if tenant in ["eu", "au", "us"]
                else "https://api.snyk.io/v1"
            ),
            rest_url=(
                f"https://api.{tenant}.snyk.io/rest"
                if tenant in ["eu", "au", "us"]
                else "https://api.snyk.io/rest"
            ),
        )
        for project in client.org_projects(org_id):
            # Extract and transform project and target data from API response
            # for rule input. Rules operate over project attributes, extended
            # with a "target" object property derived from the related target's
            # attributes.
            project_obj = {"id": project["id"]}
            project_obj.update(**project.get("attributes", {}))

            target = (
                project.get("relationships", {})
                .get("target", {})
                .get("data", {})
                .get("attributes")
            )
            if target:
                project_obj.update(target=target)

            # Clear context as this dict is (re)used in-place with each
            # execution of the project matcher rules.
            context.clear()
            component = match_fn(project_obj)
            if not component:
                # Rule did not match
                continue

            # Interpolate matcher context values, if any were extracted
            component = component.format(**context)

            have_component_tag = any(
                tag.get("value")
                for tag in project.get("attributes", {}).get("tags", [])
                if tag.get("key") == "component" and tag.get("value") == component
            )
            other_component_tags = set(
                tag.get("value")
                for tag in project.get("attributes", {}).get("tags", [])
                if tag.get("key") == "component" and tag.get("value") != component
            )

            print_format_args = {
                "dry_run": dry_run,
                "exclusive": exclusive,
                "remove": remove,
                "project": project_obj,
            }

            if exclusive:
                for other_component in other_component_tags:
                    fmtr.print(
                        action="remove other tag",
                        component=other_component,
                        **print_format_args,
                    )
                    if not dry_run:
                        client.remove_project_tag(
                            org_id,
                            project["id"],
                            tag={"key": "component", "value": other_component},
                        )

            if remove:
                if have_component_tag:
                    fmtr.print(
                        action="remove tag", component=component, **print_format_args
                    )
                    if not dry_run:
                        client.remove_project_tag(
                            org_id,
                            project["id"],
                            tag={"key": "component", "value": component},
                        )
            else:
                if not have_component_tag:
                    fmtr.print(
                        action="add tag", component=component, **print_format_args
                    )
                    if not dry_run:
                        client.add_project_tag(
                            org_id,
                            project["id"],
                            tag={"key": "component", "value": component},
                        )
                else:
                    fmtr.print(
                        action="keep tag", component=component, **print_format_args
                    )



#! /usr/bin/env python3

import logging
import re

import httpx
import typer
from rich import print
from snyk import SnykClient

from snyk_tags import __app_name__, __version__
from snyk_tags.lib.api import Api

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

logging.getLogger("httpx").setLevel(logging.WARNING)

app = typer.Typer()


# Reach to the API and generate tokens
def create_client(token: str, tenant: str) -> httpx.Client:
    base_url = (
        f"https://api.{tenant}.snyk.io/v1"
        if tenant in ["eu", "au", "us"]
        else "https://api.snyk.io/v1"
    )
    headers = {"Authorization": f"token {token}"}
    return httpx.Client(base_url=base_url, headers=headers)


# Get all organizations within a Group
def get_org_ids(token: str, group_id: str, tenant: str) -> list:
    org_ids = []
    with create_client(token=token, tenant=tenant) as client:
        req = client.get(f"group/{group_id}/orgs", timeout=None)
        if req.status_code == 404:
            logging.error(
                f"Group id: {group_id} is invalid. Error message: {req.json()}."
            )
        orgs = client.get(f"group/{group_id}/orgs").json()

        for org in orgs.get("orgs"):
            org_ids.append(org["id"])
    return org_ids


# Apply tags using the project tag API
def apply_tag_to_project(
    client: httpx.Client,
    org_id: str,
    project_id: str,
    tag: str,
    key: str,
    project_name: str,
) -> tuple:
    tag_data = {
        "key": key,
        "value": tag,
    }
    req = client.post(
        f"org/{org_id}/project/{project_id}/tags", data=tag_data, timeout=None
    )

    if req.status_code == 200:
        logging.info(f"Successfully added {tag} tag to Project: {project_name}.")
    elif req.status_code == 422:
        logging.warning(f"{tag} tag is already applied for Project: {project_name}.")
    elif req.status_code == 404:
        logging.error(
            f"Project not found, likely a READ-ONLY project. Project: {project_name}. Error message: {req.json()}."
        )
    return req.status_code, req.json()


def apply_tags_to_projects(
    token: str,
    org_ids: list,
    types: list,
    tag: str,
    key: str,
    addprojecttype: bool,
    tenant: str,
) -> None:
    with create_client(token=token, tenant=tenant) as client:
        for org_id in org_ids:
            base_url = (
                f"https://api.{tenant}.snyk.io/rest"
                if tenant in ["eu", "au", "us"]
                else "https://api.snyk.io/rest"
            )
            client_v3 = SnykClient(
                token=token, url=base_url, version="2023-08-31~experimental"
            )
            params = {"limit": 100}
            projects = client_v3.get_rest_pages(
                f"/orgs/{org_id}/projects", params=params
            )

            for project in projects:
                if project["attributes"]["type"] in types:
                    logging.debug(
                        apply_tag_to_project(
                            client=client,
                            org_id=org_id,
                            project_id=project["id"],
                            tag=tag,
                            key=key,
                            project_name=project["attributes"]["name"],
                        )
                    )
                    if addprojecttype == True:
                        logging.debug(
                            apply_tag_to_project(
                                client=client,
                                org_id=org_id,
                                project_id=project["id"],
                                tag=project["attributes"]["type"],
                                key="Type",
                                project_name=project["attributes"]["name"],
                            )
                        )


def apply_tags_to_projects_by_name(
    token: str,
    org_ids: list,
    name: str,
    ignorecase: bool,
    tag: str,
    key: str,
    tenant: str,
) -> None:
    exp = name.replace("\\", "\\\\") + "+"
    p = re.compile(exp, re.IGNORECASE) if ignorecase else re.compile(exp)
    with create_client(token=token, tenant=tenant) as client:
        for org_id in org_ids:
            base_url = (
                f"https://api.{tenant}.snyk.io/rest"
                if tenant in ["eu", "au", "us"]
                else "https://api.snyk.io/rest"
            )
            client_v3 = SnykClient(
                token=token, url=base_url, version="2023-08-31~experimental"
            )
            params = {"limit": 100}
            projects = client_v3.get_rest_pages(
                f"/orgs/{org_id}/projects", params=params
            )

            for project in projects:
                if p.search(project["attributes"]["name"]):
                    logging.debug(
                        apply_tag_to_project(
                            client=client,
                            org_id=org_id,
                            project_id=project["id"],
                            tag=tag,
                            key=key,
                            project_name=project["attributes"]["name"],
                        )
                    )


# SAST Command
sasttypes = typer.style("\n sast", bold=True, fg=typer.colors.MAGENTA)


@app.command(help="Apply Code tag to Snyk Code projects - Product:Code")
def sast(
    group_id: str = typer.Option(
        ...,  # Default value of comamand
        help="Group ID of the Snyk Group you want to apply the tags to",
        envvar=["GROUP_ID"],
    ),
    org_id: str = typer.Option(
        "",  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify one Organization ID to only apply the tag to one organization",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    sastType: str = typer.Option(
        "",  # Default value of comamand
        help=f"Type of Snyk Code projects to apply tags to: {sasttypes}",
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    addprojecttype: bool = typer.Option(
        False,
        "--addprojecttype",  # Default value of comamand
        help=f"Add an additional tag that will cover the project type e.g. Type:sast (default is false), use --addprojecttype to turn into True.",
    ),
):
    type = ["sast"] if sastType is None or sastType == "" else [sastType]
    orgs = (
        get_org_ids(snyktkn, group_id, tenant)
        if org_id is None or org_id == ""
        else [org_id]
    )
    typer.secho(
        f"\nAdding the Code tag to {type} projects in Snyk for easy filtering via the UI",
        bold=True,
    )
    apply_tags_to_projects(
        snyktkn,
        orgs,
        type,
        tag="Code",
        key="Product",
        tenant=tenant,
        addprojecttype=addprojecttype,
    )


# IaC Command
iactypes = typer.style(
    "\n terraformconfig\n terraformplan\n k8sconfig\n helmconfig\n cloudformationconfig\n armconfig",
    bold=True,
    fg=typer.colors.MAGENTA,
)


@app.command(help="Apply IaC tag to Snyk IaC projects - Product:IaC")
def iac(
    group_id: str = typer.Option(
        ...,  # Default value of comamand
        help="Group ID of the Snyk Group you want to apply the tags to",
        envvar=["GROUP_ID"],
    ),
    org_id: str = typer.Option(
        "",  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify one Organization ID to only apply the tag to one organization",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    iacType: str = typer.Option(
        "",  # Default value of comamand
        help=f"Type of Snyk IaC projects to apply tags to: {iactypes}",
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    addprojecttype: bool = typer.Option(
        False,
        "--addprojecttype",  # Default value of comamand
        help=f"Add an additional tag that will cover the project type e.g. Type:terraformplan (default is false), use --addprojecttype to turn into True.",
    ),
):
    type = (
        [
            "terraformconfig",
            "terraformplan",
            "k8sconfig",
            "helmconfig",
            "cloudformationconfig",
            "armconfig",
        ]
        if iacType is None or iacType == ""
        else [iacType]
    )
    orgs = (
        get_org_ids(snyktkn, group_id, tenant)
        if org_id is None or org_id == ""
        else [org_id]
    )
    typer.secho(
        f"\nAdding the IaC tag to {iacType} projects in Snyk for easy filtering via the UI",
        bold=True,
    )
    apply_tags_to_projects(
        snyktkn,
        orgs,
        type,
        tag="IaC",
        key="Product",
        tenant=tenant,
        addprojecttype=addprojecttype,
    )


# SCA Command
scatypes = typer.style(
    "\nmaven\n npm\n nuget\n gradle\n pip\n yarn\n gomodules\n rubygems\n composer\n sbt\n golangdep\n cocoapods\n poetry\n govendor\n cpp\n yarn-workspace\n hex\n paket\n golang",
    bold=True,
    fg=typer.colors.MAGENTA,
)


@app.command(
    help="Apply Open Source tag to Snyk Open Source projects - Product:OpenSource"
)
def sca(
    group_id: str = typer.Option(
        ...,  # Default value of comamand
        help="Group ID of the Snyk Group you want to apply the tags to",
        envvar=["GROUP_ID"],
    ),
    org_id: str = typer.Option(
        "",  # Default value of comamand,
        envvar=["ORG_ID"],
        help="Specify one Organization ID to only apply the tag to one organization",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    addprojecttype: bool = typer.Option(
        False,
        "--addprojecttype",  # Default value of comamand
        help=f"Add an additional tag that will cover the project type e.g. Type:maven (default is false), use --addprojecttype to turn into True.",
    ),
    scaType: str = typer.Option(
        "",  # Default value of comamand
        help=f"Type of Snyk Open Source projects to apply tags to (default:all): {scatypes}",
    ),
):
    type = (
        [
            "maven",
            "npm",
            "nuget",
            "gradle",
            "pip",
            "yarn",
            "gomodules",
            "rubygems",
            "composer",
            "sbt",
            "golangdep",
            "cocoapods",
            "poetry",
            "govendor",
            "cpp",
            "yarn-workspace",
            "hex",
            "paket",
            "golang",
        ]
        if scaType is None or scaType == ""
        else [scaType]
    )
    orgs = (
        get_org_ids(snyktkn, group_id, tenant)
        if org_id is None or org_id == ""
        else [org_id]
    )
    typer.secho(
        f"\nAdding the OpenSource tag to {scaType} projects in Snyk for easy filtering via the UI",
        bold=True,
    )
    apply_tags_to_projects(
        snyktkn,
        orgs,
        type,
        tag="OpenSource",
        key="Product",
        tenant=tenant,
        addprojecttype=addprojecttype,
    )


# Container Command
containertypes = typer.style(
    "\n dockerfile\n apk\n deb\n rpm\n linux", bold=True, fg=typer.colors.MAGENTA
)


@app.command(help="Apply Container tag to Snyk Container projects - Product:Container")
def container(
    group_id: str = typer.Option(
        ...,  # Default value of comamand
        help="Group ID of the Snyk Group you want to apply the tags to",
        envvar=["GROUP_ID"],
    ),
    org_id: str = typer.Option(
        "",  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify one Organization ID to only apply the tag to one organization",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    containerType: str = typer.Option(
        "",  # Default value of comamand
        help=f"Type of Snyk Container projects to apply tags to (default:all): {containertypes}",
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    addprojecttype: bool = typer.Option(
        False,
        "--addprojecttype",  # Default value of comamand
        help=f"Add an additional tag that will cover the project type e.g. Type:dockerfile (default is false), use --addprojecttype to turn into True.",
    ),
):
    type = (
        ["dockerfile", "apk", "deb", "rpm", "linux"]
        if containerType is None or containerType == ""
        else [containerType]
    )
    orgs = (
        get_org_ids(snyktkn, group_id, tenant)
        if org_id is None or org_id == ""
        else [org_id]
    )
    typer.secho(
        f"\nAdding the Container tag to {containerType} projects in Snyk for easy filtering via the UI",
        bold=True,
    )
    apply_tags_to_projects(
        snyktkn,
        orgs,
        type,
        tag="Container",
        key="Product",
        tenant=tenant,
        addprojecttype=addprojecttype,
    )


# Custom Command
@app.command(help="Apply custom tags to the preferred project type")
def custom(
    group_id: str = typer.Option(
        ...,  # Default value of comamand
        help="Group ID of the Snyk Group you want to apply the tags to",
        envvar=["GROUP_ID"],
    ),
    org_id: str = typer.Option(
        "",  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify one Organization ID to only apply the tag to one organization",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    projectType: str = typer.Option(
        ...,  # Default value of comamand
        help=f"Type of Snyk project to apply tags to (choose one): {sasttypes}, {containertypes}, {scatypes}, {iactypes}",
    ),
    tagKey: str = typer.Option(
        ..., help="Tag key: identifier of the tag"  # Default value of comamand
    ),
    tagValue: str = typer.Option(
        ..., help="Tag value: value of the tag"  # Default value of comamand
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    addprojecttype: bool = typer.Option(
        False,
        "--addprojecttype",  # Default value of comamand
        help=f"Add an additional tag that will cover the project type e.g. Type:dockerfile (default is false), use --addprojecttype to turn into True.",
    ),
):
    typer.secho(
        f"\nAdding the tag key {tagKey} and tag value {tagValue} to {projectType} projects in Snyk for easy filtering via the UI",
        bold=True,
    )

    type = []
    type.append(projectType)
    orgs = (
        get_org_ids(snyktkn, group_id, tenant)
        if org_id is None or org_id == ""
        else [org_id]
    )
    apply_tags_to_projects(
        snyktkn,
        orgs,
        type,
        tagValue,
        tagKey,
        tenant=tenant,
        addprojecttype=addprojecttype,
    )


# alltargets Command
@app.command(
    help="Apply tags at all targets on projects containing a common shared name"
)
def alltargets(
    group_id: str = typer.Option(
        ...,  # Default value of comamand
        help="Group ID of the Snyk Group you want to apply the tags to",
        envvar=["GROUP_ID"],
    ),
    org_id: str = typer.Option(
        "",  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify one Organization ID to only apply the tag to one organization",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    contains_name: str = typer.Option(
        ...,  # Default value of comamand
        help=f"Common name substring shared by projects to apply tags to",
    ),
    name_ignorecase: bool = typer.Option(
        False,
        "--name-ignorecase",  # Default value of comamand
        help=f"name case-sensitive, use --name-ignorecase to perform case-insensitive matching.",
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    tagKey: str = typer.Option(
        ..., help="Tag key: identifier of the tag"  # Default value of comamand
    ),
    tagValue: str = typer.Option(
        ..., help="Tag value: value of the tag"  # Default value of comamand
    ),
):
    typer.secho(
        f"\nAdding the tag key {tagKey} and tag value {tagValue} to {contains_name} projects in Snyk for easy filtering via the UI",
        bold=True,
    )

    orgs = (
        get_org_ids(snyktkn, group_id, tenant)
        if org_id is None or org_id == ""
        else [org_id]
    )
    apply_tags_to_projects_by_name(
        snyktkn,
        orgs,
        contains_name,
        name_ignorecase,
        tagValue,
        tagKey,
        tenant=tenant,
    )



import httpx
import backoff


def backoff_fatal_request_error(e):
    if not hasattr(e, "response") or not hasattr(e.response, "status_code"):
        # Errors which failed to get a response should retry.
        # Network failures, for example.
        return False
    if e.response.status_code == 429:
        return False
    return 400 <= e.response.status_code < 500


backoff_params = {
    "max_time": 300,
    "giveup": backoff_fatal_request_error,
    "jitter": backoff.full_jitter,
}


class Api:
    def __init__(
        self,
        token,
        v1_url="https://api.snyk.io/v1",
        rest_url="https://api.snyk.io/rest",
        rest_version="2023-07-19~beta",
    ):
        self.token = token
        self.v1_url = v1_url
        self.rest_url = rest_url
        self.rest_version = rest_version

    def v1_client(self):
        return httpx.Client(
            base_url=self.v1_url,
            headers={
                "Authorization": f"token {self.token}",
                "Content-Type": "application/json",
            },
            params={},
        )

    def v3_client(self):
        return httpx.Client(
            base_url=self.rest_url,
            headers={
                "Authorization": f"token {self.token}",
                "Content-Type": "application/vnd.api+json",
            },
            params={
                "version": self.rest_version,
            },
        )

    @backoff.on_exception(backoff.expo, httpx.HTTPError, **backoff_params)
    def org_projects(self, org_id: str):
        with self.v3_client() as c:
            next = f"/orgs/{org_id}/projects?expand=target&limit=100"
            while next:
                resp = c.get(next)
                resp.raise_for_status()
                assert resp.status_code == 200
                body = resp.json()

                projects = body.get("data", [])
                if len(projects) == 0:
                    return

                for project in body.get("data", []):
                    yield project

                next = body.get("links", {}).get("next")
            return

    @backoff.on_exception(backoff.expo, httpx.HTTPError, **backoff_params)
    def add_project_tag(self, org_id: str, project_id: str, tag: dict):
        with self.v1_client() as c:
            resp = c.post(
                f"/org/{org_id}/project/{project_id}/tags", json=tag, timeout=None
            )
            resp.raise_for_status()

    @backoff.on_exception(backoff.expo, httpx.HTTPError, **backoff_params)
    def remove_project_tag(self, org_id: str, project_id: str, tag: dict):
        with self.v1_client() as c:
            resp = c.post(
                f"/org/{org_id}/project/{project_id}/tags/remove",
                json=tag,
                timeout=None,
            )
            resp.raise_for_status()



# snyk_tags/__init__.py

__app_name__ = "snyk_tags"
__version__ = "2.5.0"

from logging import ERROR
from sre_constants import SUCCESS


(
    SUCCESS,
    ERROR,
) = range(2)

ERRORS = {ERROR: "ID Error"}



import re


def object_matcher(obj: dict, context: dict):
    for k, v in obj.items():
        if isinstance(v, str):
            return prop_string_matcher(k, v, context)
        if isinstance(v, dict):
            if "regex" in v:
                return prop_regex_matcher(k, v["regex"], context)
            else:
                return prop_object_matcher(k, v, context)


def prop_string_matcher(k: str, v: str, context: dict):
    def match_fn(obj: dict) -> bool:
        return obj.get(k) == v

    return match_fn


def prop_regex_matcher(k: str, v: str, context: dict):
    pattern = re.compile(v)

    def match_fn(obj: dict) -> bool:
        prop_val = obj.get(k)
        if not prop_val or not isinstance(prop_val, str):
            return False
        m = pattern.search(prop_val)
        if not m:
            return False
        context.update(**m.groupdict())
        return True

    return match_fn


def prop_object_matcher(k: str, v: dict, context: dict):
    prop_matcher = object_matcher(v, context)

    def match_fn(obj: dict) -> bool:
        prop_obj = obj.get(k)
        if not prop_obj or not isinstance(prop_obj, dict):
            return False
        return prop_matcher(prop_obj)

    return match_fn



#! /usr/bin/env python3

import typer

from typing import Optional
from snyk_tags import (
    __app_name__,
    __version__,
    files,
    list,
    collection,
    tag,
    remove,
    component,
)

snyk = typer.style("snyk-tags", bold=True)
snykcmd = typer.style("snyk-tags tag --help", bold=True, fg=typer.colors.MAGENTA)
snykcmd2 = typer.style("snyk-tags target --help", bold=True, fg=typer.colors.MAGENTA)
snykcmd3 = typer.style("snyk-tags list --help", bold=True, fg=typer.colors.MAGENTA)
snykcmd4 = typer.style("snyk-tags remove --help", bold=True, fg=typer.colors.MAGENTA)

app = typer.Typer(
    help=f"{snyk} helps you filter Snyk projects by adding or removing product tags and attributes to projects per product or target of projects\n\n To start using it try running:\n\n - {snykcmd} \n\n - {snykcmd2}  \n\n - {snykcmd3}  \n\n - {snykcmd4}",
    add_completion=False,
    no_args_is_help=True,
)
app.add_typer(
    tag.app,
    name="tag",
    help="Apply product/custom tags based on the product or project type",
)
app.add_typer(
    list.app, name="list", help="List all Snyk project types and all attribute types"
)
app.add_typer(
    collection.app,
    name="target",
    help="Apply custom tags and attributes to all projects within a target e.g Git repo",
)
app.add_typer(
    remove.app,
    name="remove",
    help="Remove tags from a Group and from all projects within a target e.g Git repo",
)
app.add_typer(
    files.app,
    name="fromfile",
    help="Import tags and attributes from a csv file to a target e.g. Git repo",
)
app.add_typer(
    component.app,
    name="component",
    help="Manage software component definitions with Snyk project tags",
)


def main():
    return True


def _version_callback(value: bool) -> None:
    if value:
        typer.secho(f"snyk-tags v{__version__}", bold=True)
        raise typer.Exit()


@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the application's version and exit",
        callback=_version_callback,
        is_eager=True,
    )
) -> None:
    return



import yaml
import jsonschema

from .matcher import object_matcher


project_rule_schema = {
    "type": "object",
    "properties": {
        "name": {"$ref": "#/$defs/MatcherRule"},
        "origin": {"$ref": "#/$defs/MatcherRule"},
        "target": {
            "type": "object",
            "properties": {
                "display_name": {"$ref": "#/$defs/MatcherRule"},
                "url": {"$ref": "#/$defs/MatcherRule"},
            },
        },
        "target_reference": {"$ref": "#/$defs/MatcherRule"},
    },
}

rule_schema = {
    "type": "object",
    "properties": {
        "name": {
            "type": "string",
            "minLength": 1,
        },
        "projects": {
            "type": "array",
            "items": project_rule_schema,
            "minItems": 1,
            "uniqueItems": True,
        },
        "component": {
            "type": "string",
            "minLength": 1,
        },
    },
    "required": ["name", "projects", "component"],
    "additionalProperties": False,
}

schema = {
    "type": "object",
    "properties": {
        "version": {
            "type": "number",
            "default": 1,
        },
        "rules": {
            "type": "array",
            "items": rule_schema,
            "minItems": 1,
            "uniqueItems": True,
        },
    },
    "required": ["version", "rules"],
    "additionalProperties": False,
    "$defs": {
        "MatcherRule": {
            "oneOf": [
                {
                    "type": "string",
                    "minLength": 1,
                },
                {
                    "type": "object",
                    "properties": {
                        "regex": {
                            "type": "string",
                            "minLength": 1,
                        },
                    },
                    "required": ["regex"],
                },
            ],
        },
    },
}


def parse_rules(input):
    data = yaml.safe_load(input)
    jsonschema.validate(data, schema)
    return data


def project_matcher(data):
    context = {}
    rule_matchers = []
    for rule in data["rules"]:
        match_fns = [object_matcher(project, context) for project in rule["projects"]]
        rule_matchers.append((match_fns, rule["component"]))

    def match_fn(obj: dict) -> str:
        for project_match_fns, component in rule_matchers:
            if any([match_fn(obj) for match_fn in project_match_fns]):
                return component
        return None

    return (match_fn, context)



#! /usr/bin/env python3
import logging
import re

import httpx
import typer
from rich import print
from snyk import SnykClient

app = typer.Typer()

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

logging.getLogger("httpx").setLevel(logging.WARNING)


# Remove tags from a specific project
def remove_tag_from_project(
    token: str,
    org_id: str,
    project_id: str,
    tag: str,
    key: str,
    project_name: str,
    tenant: str,
) -> tuple:
    base_url = (
        f"https://api.{tenant}.snyk.io/v1"
        if tenant in ["eu", "au", "us"]
        else "https://api.snyk.io/v1"
    )
    client = SnykClient(token=token, url=base_url)
    try:
        client.organizations.get(org_id).projects.get(project_id).tags.delete(key, tag)
        print(f"Removing tag {key}:{tag} from {project_name}")
    except Exception as e:
        if "422" in str(e):
            print(
                f"The tag {key}:{tag} has already been removed from Project: {project_name}"
            )
        elif "404" in str(e):
            print((f"Project not found. Project: {project_name}. Error message: {e}."))
        else:
            print(f"Unknown error {e}")


# Remove tag loop with pysnyk
def remove_tags_from_projects(
    token: str, org_id: list, name: str, tag: str, key: str, tenant: str
) -> None:
    base_url = (
        f"https://api.{tenant}.snyk.io/rest"
        if tenant in ["eu", "au", "us"]
        else "https://api.snyk.io/rest"
    )
    client_v3 = SnykClient(token=token, url=base_url, version="2023-08-31~experimental")
    params = {"limit": 100}
    projects = client_v3.get_rest_pages(f"/orgs/{org_id}/projects", params=params)

    isname = 0
    for project in projects:
        if project["attributes"]["name"].startswith(name):
            remove_tag_from_project(
                token=token,
                org_id=org_id,
                project_id=project["id"],
                tag=tag,
                key=key,
                tenant=tenant,
                project_name=project["attributes"]["name"],
            )
        else:
            isname = 1
    if isname == 1:
        print(
            f"[bold red]{name}[/bold red] is not a valid target, please check it is a target within the organization e.g. [bold blue]snyk-labs/snyk-goof[/bold blue]"
        )


def remove_tags_from_projects_by_name(
    token: str,
    org_id: str,
    name: str,
    ignorecase: bool,
    tag: str,
    key: str,
    tenant: str,
) -> None:
    exp = name.replace("\\", "\\\\") + "+"
    p = re.compile(exp, re.IGNORECASE) if ignorecase else re.compile(exp)
    base_url = (
        f"https://api.{tenant}.snyk.io/rest"
        if tenant in ["eu", "au", "us"]
        else "https://api.snyk.io/rest"
    )
    client_v3 = SnykClient(token=token, url=base_url, version="2023-08-31~experimental")
    params = {"limit": 100}
    projects = client_v3.get_rest_pages(f"/orgs/{org_id}/projects", params=params)

    for project in projects:
        if p.search(project["attributes"]["name"]):
            remove_tag_from_project(
                token=token,
                org_id=org_id,
                project_id=project["id"],
                tag=tag,
                key=key,
                project_name=project["attributes"]["name"],
                tenant=tenant,
            )


# Reach to the API and generate tokens
def create_client(token: str, tenant: str) -> httpx.Client:
    base_url = (
        f"https://api.{tenant}.snyk.io/v1"
        if tenant in ["eu", "au", "us"]
        else "https://api.snyk.io/v1"
    )
    headers = {"Authorization": f"token {token}"}
    return httpx.Client(base_url=base_url, headers=headers)


# Apply tags to a specific project
def remove_tag_from_group(
    token: str, group_id: str, force: bool, tag: str, key: str, tenant: str
) -> tuple:
    if force is True:
        tag_data = {"key": key, "value": tag, "force": force}
    else:
        tag_data = {"key": key, "value": tag}

    with create_client(token=token, tenant=tenant) as client:
        req = client.post(f"group/{group_id}/tags/delete", data=tag_data, timeout=None)
        group = client.get(f"group/{group_id}/orgs").json()
        group_name = group["name"]

        if req.status_code == 200:
            print(f"Successfully removed {key}:{tag} from Group: {group_name}")
        elif req.status_code == 403:
            print(
                f"The tag {key}:{tag} has entities attached in Group: {group_name}  Error message: {req.json()}"
            )
        elif req.status_code == 422:
            print(
                f"The tag {key}:{tag} has already been removed from Group: {group_name}  Error message: {req.json()}"
            )
        elif req.status_code == 404:
            print(
                f"Tag {key}:{tag} not found in {group_name}. Error message: {req.json()}."
            )
        return req.status_code, req.json()


repoexample = typer.style("'snyk-labs/nodejs-goof'", bold=True, fg=typer.colors.MAGENTA)


@app.command(help=f"Remove a tag from a target, for example {repoexample}")
def tag_from_target(
    org_id: str = typer.Option(
        ...,  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify the Organization ID to remove the tag from",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    target: str = typer.Option(
        ...,  # Default value of comamand
        help=f"Name of the target, for example {repoexample}",
    ),
    tagKey: str = typer.Option(
        ..., help="Tag key: identifier of the tag"  # Default value of comamand
    ),
    tagValue: str = typer.Option(
        ..., help="Tag value: value of the tag"  # Default value of comamand
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
):
    typer.secho(
        f"\nRemoving {tagKey}:{tagValue} from projects within {target}", bold=True
    )
    remove_tags_from_projects(snyktkn, org_id, target, tagValue, tagKey, tenant)


@app.command(
    help=f"Remove a tag from all targets, on projects containing a common shared name"
)
def tag_from_alltargets(
    org_id: str = typer.Option(
        ...,  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify the Organization ID to remove the tag from",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    contains_name: str = typer.Option(
        ...,  # Default value of comamand
        help=f"Common name substring shared by projects to remove tags",
    ),
    name_ignorecase: bool = typer.Option(
        False,
        "--name-ignorecase",  # Default value of comamand
        help=f"name case-sensitive, use --name-ignorecase to perform case-insensitive matching.",
    ),
    tagKey: str = typer.Option(
        ..., help="Tag key: identifier of the tag"  # Default value of comamand
    ),
    tagValue: str = typer.Option(
        ..., help="Tag value: value of the tag"  # Default value of comamand
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
):
    typer.secho(
        f"\nRemoving {tagKey}:{tagValue} from projects within {org_id}", bold=True
    )
    remove_tags_from_projects_by_name(
        snyktkn, org_id, contains_name, name_ignorecase, tagValue, tagKey, tenant
    )


@app.command(help=f"Remove a tag from a Group, this can be forced through --force")
def tag_from_group(
    group_id: str = typer.Option(
        ...,  # Default value of comamand
        envvar=["GROUP_ID"],
        help="Specify the Group where you want to remove the tag from",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with Group admin access",
        envvar=["SNYK_TOKEN"],
    ),
    force: bool = typer.Option(
        False,
        "--force",  # Default value of comamand
        help=f"Force delete tag that has entities (default is false), use --force to turn into True.",
    ),
    tagKey: str = typer.Option(
        ..., help="Tag key: identifier of the tag"  # Default value of comamand
    ),
    tagValue: str = typer.Option(
        ..., help="Tag value: value of the tag"  # Default value of comamand
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
):
    typer.secho(f"\nRemoving {tagKey}:{tagValue} from Group ID: {group_id}", bold=True)
    remove_tag_from_group(snyktkn, group_id, force, tagValue, tagKey, tenant)



#! /usr/bin/env python3
import typer
from pathlib import Path
from typing import List, Optional
import csv
from snyk_tags import collection, attribute, remove
from rich import print
import json

app = typer.Typer()
repoexample = typer.style("'snyk-labs/nodejs-goof'", bold=True, fg=typer.colors.MAGENTA)
tagexample = typer.style("org-id,target,key,value", bold=True, fg=typer.colors.MAGENTA)
attributesexample = typer.style(
    "org-id,target,criticality,environment,lifecycle",
    bold=True,
    fg=typer.colors.MAGENTA,
)
removetaggroupexample = typer.style("key,value", bold=True, fg=typer.colors.MAGENTA)
removetagtargetexample = typer.style(
    "org-id,target,key,value", bold=True, fg=typer.colors.MAGENTA
)


@app.command(
    help=f"Apply a custom tag from a .csv or .json to a target, for example {repoexample} \n\n The .csv or .json must be in the format {tagexample}"
)
def target_tag(
    file: List[Path] = typer.Option(
        ..., help=f".csv or .json file with the format {tagexample}"
    ),
    snyktkn: str = typer.Option(
        ..., help="Snyk API token with org admin access", envvar=["SNYK_TOKEN"]
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
):
    for path in file:
        filters = {}
        if path.is_file():
            openfile = open(path)
            if ".csv" in openfile.name:
                csvreader = csv.DictReader(openfile)
                for row in csvreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    key = row.get("key")
                    value = row.get("value")
                    filters = {
                        attr: val
                        for attr, val in row.items()
                        if attr in ["target_reference", "origins", "types"]
                    }
                    typer.secho(
                        f"\nAdding the tag key {key} and tag value {value} to projects within {target} for easy filtering via the UI",
                        bold=True,
                    )
                    collection.apply_tags_to_projects(
                        snyktkn, [org_id], target, value, key, tenant, filters
                    )
                openfile.close()
            elif ".json" in openfile.name:
                jsonreader = json.load(openfile)
                for row in jsonreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    key = row.get("key")
                    value = row.get("value")
                    filters = {
                        attr: val
                        for attr, val in row.items()
                        if attr in ["target_reference", "origins", "types"]
                    }
                    typer.secho(
                        f"\nAdding the tag key {key} and tag value {value} to projects within {target} for easy filtering via the UI",
                        bold=True,
                    )
                    collection.apply_tags_to_projects(
                        snyktkn, [org_id], target, value, key, tenant, filters
                    )
                openfile.close()
            else:
                print(
                    f"The file {openfile.name} is not valid, it must be either a .csv or a .json"
                )
                openfile.close()
        else:
            print(f"The file or path does not exist")


@app.command(
    help=f"Apply attributes from a .csv or .json to a target, for example {repoexample} \n\n The .csv or .json must be in the format {attributesexample}"
)
def target_attributes(
    file: List[Path] = typer.Option(
        ..., help=f".csv or .json file with the format {attributesexample}"
    ),
    snyktkn: str = typer.Option(
        ..., help="Snyk API token with org admin access", envvar=["SNYK_TOKEN"]
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
):
    for path in file:
        filters = {}
        if path.is_file():
            openfile = open(path)
            if ".csv" in openfile.name:
                csvreader = csv.DictReader(openfile)
                for row in csvreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    criticality = row.get("criticality")
                    environment = row.get("environment")
                    lifecycle = row.get("lifecycle")
                    filters = {
                        attr: val
                        for attr, val in row.items()
                        if attr in ["target_reference", "origins", "types"]
                    }
                    typer.secho(
                        f"\nAdding the attributes {criticality}, {environment} and {lifecycle} to projects within {target} for easy filtering via the UI",
                        bold=True,
                        fg=typer.colors.MAGENTA,
                    )
                    attribute.apply_attributes_to_projects(
                        snyktkn,
                        [org_id],
                        target,
                        [criticality],
                        [environment],
                        [lifecycle],
                        tenant,
                        filters,
                    )
                openfile.close()
            elif ".json" in openfile.name:
                jsonreader = json.load(openfile)
                for row in jsonreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    criticality = row.get("criticality")
                    environment = row.get("environment")
                    lifecycle = row.get("lifecycle")
                    filters = {
                        attr: val
                        for attr, val in row.items()
                        if attr in ["target_reference", "origins", "types"]
                    }
                    typer.secho(
                        f"\nAdding the attributes {criticality}, {environment} and {lifecycle} to projects within {target} for easy filtering via the UI",
                        bold=True,
                        fg=typer.colors.MAGENTA,
                    )
                    attribute.apply_attributes_to_projects(
                        snyktkn,
                        [org_id],
                        target,
                        [criticality],
                        [environment],
                        [lifecycle],
                        tenant,
                        filters,
                    )
                openfile.close()
            else:
                print(
                    f"The file {openfile.name} is not valid, it must be either a .csv or a .json"
                )
                openfile.close()
        else:
            print(f"The file or path does not exist")


@app.command(
    help=f"Remove tags from a Group with .csv or .json, this can be forced through --force"
)
def remove_tag_from_group(
    file: List[Path] = typer.Option(
        ..., help=f".csv or .json file with the format {removetaggroupexample}"
    ),
    group_id: str = typer.Option(
        ...,
        envvar=["GROUP_ID"],
        help="Specify the Group where you want to remove the tag from",
    ),
    snyktkn: str = typer.Option(
        ..., help="Snyk API token with Group admin access", envvar=["SNYK_TOKEN"]
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    force: bool = typer.Option(
        False,
        "--force",
        help=f"Force delete tag that has entities (default is false), use --force to turn into True.",
    ),
):
    for path in file:
        if path.is_file():
            openfile = open(path)
            if ".csv" in openfile.name:
                csvreader = csv.DictReader(openfile)
                for row in csvreader:
                    tagKey = row.get("key")
                    tagValue = row.get("value")
                    typer.secho(
                        f"\nRemoving {tagKey}:{tagValue} from Group ID: {group_id}",
                        bold=True,
                    )
                    if tagKey and tagValue:
                        remove.remove_tag_from_group(
                            snyktkn, group_id, force, tagValue, tagKey, tenant
                        )
                openfile.close()
            elif ".json" in openfile.name:
                jsonreader = json.load(openfile)
                for row in jsonreader:
                    tagKey = row.get("key")
                    tagValue = row.get("value")
                    typer.secho(
                        f"\nRemoving {tagKey}:{tagValue} from Group ID: {group_id}",
                        bold=True,
                    )
                    if tagKey and tagValue:
                        remove.remove_tag_from_group(
                            snyktkn, group_id, force, tagValue, tagKey, tenant
                        )
                openfile.close()
            else:
                print(
                    f"The file {openfile.name} is not valid, it must be either a .csv or a .json"
                )
                openfile.close()
        else:
            print(f"The file or path does not exist")


@app.command(
    help=f"Remove a tag from a target with .csv or .json, for example {repoexample}"
)
def remove_tag_from_target(
    file: List[Path] = typer.Option(
        ..., help=f".csv or .json file with the format {removetagtargetexample}"
    ),
    snyktkn: str = typer.Option(
        ..., help="Snyk API token with org admin access", envvar=["SNYK_TOKEN"]
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
):
    for path in file:
        if path.is_file():
            openfile = open(path)
            if ".csv" in openfile.name:
                csvreader = csv.DictReader(openfile)
                for row in csvreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    key = row.get("key")
                    value = row.get("value")
                    typer.secho(
                        f"\nRemoving {key}:{value} from projects within {target}",
                        bold=True,
                    )
                    remove.remove_tags_from_projects(
                        snyktkn, org_id, target, value, key, tenant
                    )
                openfile.close()
            elif ".json" in openfile.name:
                jsonreader = json.load(openfile)
                for row in jsonreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    key = row.get("key")
                    value = row.get("value")
                    typer.secho(
                        f"\nRemoving {key}:{value} from projects within {target}",
                        bold=True,
                    )
                    remove.remove_tags_from_projects(
                        snyktkn, org_id, target, value, key, tenant
                    )
                openfile.close()
            else:
                print(
                    f"The file {openfile.name} is not valid, it must be either a .csv or a .json"
                )
                openfile.close()
        else:
            print(f"The file or path does not exist")



#! /usr/bin/env python3

import logging
import re

import httpx
import typer
import validators
from github import Github
from github import Auth
from rich import print
from snyk import SnykClient

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

logging.getLogger("httpx").setLevel(logging.WARNING)

app = typer.Typer()


# Reach to the API and generate tokens
def create_client(token: str, tenant: str) -> httpx.Client:
    base_url = (
        f"https://api.{tenant}.snyk.io/v1"
        if tenant in ["eu", "au", "us"]
        else "https://api.snyk.io/v1"
    )
    headers = {"Authorization": f"token {token}"}
    return httpx.Client(base_url=base_url, headers=headers)


# Apply tags to a specific project
def apply_tag_to_project(
    client: httpx.Client,
    org_id: str,
    project_id: str,
    tag: str,
    key: str,
    project_name: str,
) -> tuple:
    tag_data = {
        "key": key,
        "value": tag,
    }

    req = client.post(
        f"org/{org_id}/project/{project_id}/tags", data=tag_data, timeout=None
    )

    if req.status_code == 200:
        logging.info(f"Successfully added {tag_data} tags to Project: {project_name}.")
    elif req.status_code == 422:
        logging.warning(
            f"Tag {key}:{tag} is already applied for Project: {project_name}."
        )
    elif req.status_code == 404:
        logging.error(
            f"Project not found, likely a READ-ONLY project. Project: {project_name}. Error message: {req.json()}."
        )
    return req.status_code, req.json()


def validate_gh_url(url: str) -> str:
    validated_url = "invalid_gh_base_url"
    if validators.url(url):
        validated_url = url
    return validated_url


# GitHub Tagging Loop
def apply_github_owner_to_repo(
    snyktoken: str,
    org_ids: list,
    name: str,
    githubtoken: str,
    tenant: str,
    gh_base_url: str,
) -> None:
    ghauth = Auth.Token(githubtoken)
    g = Github(base_url=gh_base_url, auth=ghauth)
    with create_client(token=snyktoken, tenant=tenant) as client:
        for org_id in org_ids:
            base_url = (
                f"https://api.{tenant}.snyk.io/rest"
                if tenant in ["eu", "au", "us"]
                else "https://api.snyk.io/rest"
            )
            client_v3 = SnykClient(
                token=snyktoken, url=base_url, version="2023-08-31~experimental"
            )
            params = {"limit": 100}
            projects = client_v3.get_rest_pages(
                f"/orgs/{org_id}/projects", params=params
            )

            badname = 0
            rightname = 0
            for project in projects:
                if project["attributes"]["name"].startswith(name + "(") or project[
                    "attributes"
                ]["name"].startswith(name + ":"):
                    repo = g.get_repo(name)
                    contents = [""]
                    while contents:
                        entries = repo.get_contents(contents.pop(0))
                        for file_content in entries:
                            if file_content.type == "dir":
                                contents.append(file_content.path)
                            elif "CODEOWNERS" in file_content.path:
                                decoded = file_content.decoded_content.decode("utf-8")
                                if "@" in decoded:
                                    lines = re.split("\n| ", decoded)
                                    for word in lines:
                                        owner = word
                                        if owner == "":
                                            pass
                                        elif owner[0] == "@":
                                            owner = owner[1:]
                                            apply_tag_to_project(
                                                client=client,
                                                org_id=org_id,
                                                project_id=project["id"],
                                                tag=owner,
                                                key="Owner",
                                                project_name=project["attributes"][
                                                    "name"
                                                ],
                                            )
                                else:
                                    print("Invalid CODEOWNERS file")
                                break
                    rightname = 1
                else:
                    badname = 1
            if badname == 1 and rightname == 0:
                print(
                    f"[bold red]{name}[/bold red] is not a valid target, please check it is a target within the organization e.g. [bold blue]snyk-labs/snyk-goof[/bold blue]"
                )


def apply_github_topics_to_repo(
    snyktoken: str,
    org_ids: list,
    name: str,
    githubtoken: str,
    tenant: str,
    gh_base_url: str,
) -> None:
    ghauth = Auth.Token(githubtoken)
    g = Github(base_url=gh_base_url, auth=ghauth)
    with create_client(token=snyktoken, tenant=tenant) as client:
        for org_id in org_ids:
            base_url = (
                f"https://api.{tenant}.snyk.io/rest"
                if tenant in ["eu", "au", "us"]
                else "https://api.snyk.io/rest"
            )
            client_v3 = SnykClient(
                token=snyktoken, url=base_url, version="2023-08-31~experimental"
            )
            params = {"limit": 100}
            projects = client_v3.get_rest_pages(
                f"/orgs/{org_id}/projects", params=params
            )

            badname = 0
            rightname = 0
            for project in projects:
                if project["attributes"]["name"].startswith(name + "(") or project[
                    "attributes"
                ]["name"].startswith(name + ":"):
                    repo = g.get_repo(name)
                    if repo.get_topics() == []:
                        print(
                            f"[bold red]{name}[/bold red] does not have valid topics, please check the repository has valid topics"
                        )
                        break
                    else:
                        for topic in repo.get_topics():
                            apply_tag_to_project(
                                client=client,
                                org_id=org_id,
                                project_id=project["id"],
                                tag=topic,
                                key="GitHubTopic",
                                project_name=project["attributes"]["name"],
                            )
                    rightname = 1
                else:
                    badname = 1
            if badname == 1 and rightname == 0:
                print(
                    f"[bold red]{name}[/bold red] is not a valid target, please check it is a target within the organization e.g. [bold blue]snyk-labs/snyk-goof[/bold blue]"
                )


repoexample = typer.style("'snyk-labs/nodejs-goof'", bold=True, fg=typer.colors.MAGENTA)


# GitHub Code Owner Tagging
@app.command(
    help=f"Add the GitHub CODEOWNERS (only GitHub handles) as tags to the specified repo in Snyk, for example {repoexample}"
)
def owners(
    org_id: str = typer.Option(
        ...,  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify the Organization ID where you want to apply the tag",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    target: str = typer.Option(
        ...,  # Default value of comamand
        help=f"Name of the repo, for example {repoexample}",
    ),
    githubtkn: str = typer.Option(
        ...,  # Default value of comamand
        help="GitHub Personal Access Token with access to the repository",
        envvar=["GITHUB_TOKEN"],
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    gh_base_url: str = typer.Option(
        "https://api.github.com",
        help=f"Base URL of Github instance (e.g. https://ghe.internal/api/v3). Defaults to https://api.github.com (Github.com)",
    ),
):
    typer.secho(
        f"\nAdding the Owner tag to projects within {target} for easy filtering via the UI",
        bold=True,
        fg=typer.colors.MAGENTA,
    )
    gh_base_url = validate_gh_url(gh_base_url)
    apply_github_owner_to_repo(
        snyktkn, [org_id], target, githubtkn, tenant=tenant, gh_base_url=gh_base_url
    )


# GitHub Topics Tagging
@app.command(
    help=f"Add the GitHub repo's Topics as tags to the specified repo in Snyk, for example {repoexample}"
)
def topics(
    org_id: str = typer.Option(
        ...,  # Default value of comamand
        envvar=["ORG_ID"],
        help="Specify the Organization ID where you want to apply the tag",
    ),
    snyktkn: str = typer.Option(
        ...,  # Default value of comamand
        help="Snyk API token with org admin access",
        envvar=["SNYK_TOKEN"],
    ),
    target: str = typer.Option(
        ...,  # Default value of comamand
        help=f"Name of the repo, for example {repoexample}",
    ),
    githubtkn: str = typer.Option(
        ...,  # Default value of comamand
        help="GitHub Personal Access Token with access to the repository",
        envvar=["GITHUB_TOKEN"],
    ),
    tenant: str = typer.Option(
        "",  # Default value of comamand
        help=f"Defaults to US tenant (app.snyk.io), add 'eu', 'au' or 'us' to use alternative regional tenant. Use --tenant to change tenant.",
    ),
    gh_base_url: str = typer.Option(
        "https://api.github.com",
        help=f"Base URL of Github instance (e.g. https://ghe.internal/api/v3). Defaults to https://api.github.com (Github.com)",
    ),
):
    typer.secho(
        f"\nAdding the GitHubTopic tag to projects within {target} for easy filtering via the UI",
        bold=True,
        fg=typer.colors.MAGENTA,
    )
    gh_base_url = validate_gh_url(gh_base_url)
    apply_github_topics_to_repo(
        snyktkn, [org_id], target, githubtkn, tenant=tenant, gh_base_url=gh_base_url
    )
